function [cj,eta,xi]=remes2(fun,approx,ab,xi)
% Multiple exchange algorithm
% [cj,eta,xi]=remes2(fun,approx,ab,xi)    
% [cj,eta,xi]=remes2(fun,approx,ab,cj)    
% cj:     parameters of function (row vector)
% eta:    maximum error
% xi:     critical points (column vector)
% ab:     interval of interest
% fun     feval(fun,x) or eval(f) should evaluate a function at the points x 
% approx: feval(approx,x,c) or eval(approx) should evaluate a function that
%         depends linearly on c at the points x

% DPL 2003-05-14
% 1. Code should be made more robust against possible equality, e.g. at
% present the sequence [1 2 2 1] is not recognized as having an extremum.
% 2. Current stopping rule is even less robust.
% 3. Better care should be taken that alternating property holds

fknown=exist(fun); aknown=exist(approx);
nab=257; xab=linspace(ab(1),ab(2),nab)';
if fknown, f=feval(fun,xi); else x=xab; fab=eval(fun); endif 
cknown=size(xi,1)==1; 
if cknown, cj=xi; n=length(cj); xi=zeros(n+1,1); 
  else n=length(xi)-1; cj=zeros(1,n); 
endif
M=zeros(n+1,n+1); M(:,n+1)=cumprod(-ones(n+1,1)); eta0=0; 

while 1,
  if cknown, 
    c=cj; 
    if aknown, a=feval(approx,xab,c); else x=xab; a=eval(approx); endif
    aix=fab-a;
    k=2:nab-1; crit=find((aix(k)-aix(k+1)).*(aix(k)-aix(k-1))>0); 
    goodsign=find(aix(crit).*(aix(crit-1)-2*aix(crit)+aix(crit+1))<0);
    if length(goodsign)>=n-1, crit=crit(goodsign); end
    lc=length(crit);
    if lc<n-1,
       error("remes2: can't continue, too few extrema");
    elseif lc>n+1,  
%     warning("remes2: too many extrema, discarding weakest");
      [as,js]=sort(abs(aix(crit))); 
      crit(js(1:lc-n-1))=[]; lc=n+1;
    else
      s1=sign(aix(crit(1))); s2=sign(aix(crit(lc)));
      if aix(1)*s1<aix(nab)*s2, take=1; else take=2; endif
    end
    xi=zeros(lc,1); 
    for k=1:lc,  
      ix=crit(k)+(-1:1); x=xab(ix); ax=aix(ix);
      while 1,
        xnew=extremum1(x,ax); err=abs(xnew(2)-x(2));
        if err<1e-8, break; endif
        x=xnew;
        if fknown, f=feval(fun,x); else f=eval(fun); endif
        if aknown, a=feval(approx,x,cj); else a=eval(approx); endif
        ax=f-a;
      end
      xi(k)=x(2); 
    end
    if lc==n-1, xi=[ab(1);xi;ab(2)]; endif;
    if lc==n, if take==1, xi=[ab(1);xi]; else xi=[xi;ab(2)]; endif, endif
  endif
  x=xi; cknown=1; 
  if fknown, b=feval(fun,x); else b=eval(fun); endif
  for j=1:n, 
     c=(j==(1:n));  
     if aknown, a=feval(approx,xab,c); else x=xi; a=eval(approx); endif  
     M(:,j)=a;
  end
  y=M\b; cj=y(1:n).'; eta=abs(y(n+1)); 
  if eta<=eta0, break, endif
  eta0=eta;
end 

