clear

moment=2;  % Number of seconds to pause in slide show mode

% A first look
deg=1:360; t=pi*deg/180; z=exp(1i*t); g=isgamma(z);
axis([0 360]); gset xtics 0,30,360; gset nokey;
clg; title("Real and imaginary parts of 1/Gamma(exp(i*theta))");
xlabel("Theta in degrees");
plot (deg,real(g),'1');
hold on; plot (deg,imag(g),'2');
print("rigamma.ps")
if exist('interactive'),
  prompt("Current graph saved as rigamma.ps"); 
else pause(moment)
end;

% Fourier approximation
c0=[0,1,0.5772,-0.6559];
clg
q=c0(1)+c0(2)*z+c0(3)*z.^2+c0(4)*z.^3; 
title("Absolute and component error of degree 3 Fourier approximation");
plot(deg,abs(g-q),'1')
plot(deg,-abs(g-q),'1')
plot(deg,real(g-q),'2')
plot(deg,imag(g-q),'3')
print("errorfourier.ps");
if exist('interactive'),
  prompt("Current graph saved as errorfourier.ps"); 
else pause(moment);
end

realfun="real(isgamma(exp(i*x)));";
imagfun="imag(isgamma(exp(i*x)));";
realapprox="cos(x*(0:3))*c(:);";
imagapprox="sin(x*(1:3))*c(:);";
shiftimagapprox="sin(x*(1:3))*c(:);";
[cr,etar,xjr]=remes2(realfun, realapprox, [0,pi], c0(1:4))
[ci,etai,xji]=remes2(imagfun, imagapprox, [0,pi], c0(2:4))

q=cr(1)+cr(2)*z+cr(3)*z.^2+cr(4)*z.^3; 
clg;
title("Absolute and component error of best approximation of real part");
plot(deg,abs(g-q),'1')
plot(deg,-abs(g-q),'1')
plot(deg,real(g-q),'2')
plot(deg,imag(g-q),'3')
print("errorreal.ps");
if exist('interactive'),
  prompt("Current graph saved as errorreal.ps"); 
else pause(moment);
end

q=ci(1)*z+ci(2)*z.^2+ci(3)*z.^3; 
clg;
title("Absolute and component error of best approximation of imaginary part");
plot(deg,abs(g-q),'1')
plot(deg,-abs(g-q),'1')
plot(deg,real(g-q),'2')
plot(deg,imag(g-q),'3')
print("errorimag.ps");
error_imag=max(abs(g-q))
if exist('interactive'),
  prompt("Current graph saved as errorimag.ps"); 
else pause(moment);
end;

i=2:180; cs=0;  
for k=1:5,
  er=abs(g-q-cs); ex=find((er(i)-er(i-1)).*(er(i)-er(i+1))>0)+1;
  er(ex); dcs=(ans(1)-ans(length(ans)))/2; Dcs(k)=dcs;
  cs=cs+dcs; Cs(k)=cs; if k==1 | k==5, maxerr=[ex; er(ex)], end
end
Dcs, cs

q=q+cs;
clg;
title("Absolute and component error of improved approximation");
plot(deg,abs(g-q),'1')
plot(deg,-abs(g-q),'1')
plot(deg,real(g-q),'2')
plot(deg,imag(g-q),'3')
print("errorimproved.ps");
error_improved=max(abs(g-q))
if exist('interactive'),
  prompt("Current graph saved as errorimproved.ps"); 
else pause(moment);
end

function [y,c]=epsilon(th)
  z=z=exp(i*th); z=[z;-1;conj(z)]; [c,e]=p5solve(z); y=real(e); 
endfunction

th=[65;113]*pi/180; f0=epsilon(th); format long;
while true,
  % Numerical Newton iteration
  dth=hessian('epsilon',th)\cjac('epsilon',th)'; new=th-dth; f1=epsilon(new);
  if abs(f1-f0)<1e-15, break; end
  while true, th1_th2_eps=[new' f1],  if f1>f0, break; end    
    disp('Emergency damped step');
    dth=dth/2; new=th-dth; f1=epsilon(new);
  end
  f0=f1; th=new;
  end

[y,c]=epsilon(th); q=c(1)+c(2)*z+c(3)*z.^2+c(4)*z.^3; 
clg;
title("Absolute and component error of best approximation");
plot(deg,abs(g-q),'1')
plot(deg,-abs(g-q),'1')
plot(deg,real(g-q),'2')
plot(deg,imag(g-q),'3')
print("errorbest.ps");
best_error=max(abs(g-q))
if exist('interactive'),
  prompt("Current graph saved as errorbest.ps"); 
else pause(moment);
end

